//
//  YHFamilyMemberFormVC.swift
//  galaxy
//
//  Created by edy on 2024/1/22.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit

enum YHFamilyMemberEditType:Int {
    case none = 1
    case canEdit = 2
    case canCancel = 3
}

class YHFamilyMemberFormVC: YHBaseViewController {

    var familyMemberInfo: YHFamilyMemberInfo?
    var isChildsEditMode: YHFamilyMemberEditType = .none
    var isBrothersEditMode: YHFamilyMemberEditType = .none
    
    var items:[[YHFamilyMemberProtocol]] = []
    
    lazy var tableView: UITableView = {
        
        let tableView = UITableView(frame:.zero, style:.plain)

        if #available(iOS 11.0, *) {
            tableView.contentInsetAdjustmentBehavior = .never
        }
        tableView.backgroundColor = .clear
        tableView.separatorStyle = .none
        tableView.delegate = self
        tableView.dataSource = self
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: "UITableViewCell")
        tableView.register(YHDoubleChoiceFormCell.self, forCellReuseIdentifier: YHDoubleChoiceFormCell.cellReuseIdentifier)
        tableView.register(YHSingleLineInputCell.self, forCellReuseIdentifier: YHSingleLineInputCell.cellReuseIdentifier)
        tableView.register(YHSingleLineSelectCell.self, forCellReuseIdentifier: YHSingleLineSelectCell.cellReuseIdentifier)
        tableView.register(YHFormTitleCell.self, forCellReuseIdentifier: YHFormTitleCell.cellReuseIdentifier)
        tableView.register(YHFormAddInfoCell.self, forCellReuseIdentifier: YHFormAddInfoCell.cellReuseIdentifier)
        return tableView
    }()
    
    lazy var submitBtn:UIButton = {
        let btn = UIButton.bs_button(title: "提交".local, font: UIFont.PFSCR(ofSize: 14), normalColor: .white)
        btn.backgroundColor = UIColor(hexString:"#4388FF")
        btn.layer.cornerRadius = 8.0
        btn.addTarget(self, action: #selector(didClickSubmitBtn), for: .touchUpInside)
        return btn
    }()
    
    lazy var bottomView:UIView = {
        let view = UIView()
        view.backgroundColor = .white
        view.addSubview(submitBtn)
        return view
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.gk_navTitle = "家庭成员信息填写"
        view.backgroundColor = UIColor(hexString:"#F8F8F8")
        createUI()
        loadData()
        
    }
    
    func createUI() {
        
        view.addSubview(tableView);
        view.addSubview(bottomView)
        
        let submitBtnHeight = 48.0
        let gap = 8.0
        bottomView.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.bottom.equalTo(view)
            make.height.equalTo(submitBtnHeight+gap*2+k_Height_safeAreaInsetsBottom())
        }
        submitBtn.snp.makeConstraints { make in
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
            make.height.equalTo(48)
            make.top.equalTo(8)
        }
        
        tableView.snp.makeConstraints { make in
            make.top.equalToSuperview().offset(k_Height_NavigationtBarAndStatuBar)
            make.bottom.equalTo(bottomView.snp.top)
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
        }
    }
    
    @objc func didClickSubmitBtn() {
        
    }
    
    func loadData() {
        
        let dic = [
            "father": [
                "id": 0,
                "deceased": 0,
                "custody": 100,
                "nationality": "",
                "order_id": 0,
                "relation": 0,
                "has_degree": 0,
                "has_degree_json": [],
                "follow": 0,
                "statement": 0,
                "subset_name": "",
                "subset_name_pinyin": "",
                "used_name": "",
                "birthday": "",
                "birth_place_aboard": 0,
                "birth_place": [],
                "occupation": "",
                "occupation_name": "",
                "operator_ame": "",
                "child_has_hk_id": [],
                "hk_identity_card": "",
                "sex": 0,
                "married": 0,
                "nows": 0,
                "address": [],
                "child_stepchild": [],
                "child_in_hk": [],
                "country_identity": [],
                "hk_identity": "",
                "hk_identity_other": "",
                "live_country": "",
                "is_handled": 0,
                "finish_follow": 0,
                "step": 0,
                "not_fill_num": -1,
                "certificates": [],
                "is_live_oversea_year": 0
            ],
            "mother": [
                "id": 0,
                "deceased": 0,
                "custody": 100,
                "nationality": "",
                "order_id": 0,
                "relation": 0,
                "has_degree": 0,
                "has_degree_json": [],
                "follow": 0,
                "statement": 0,
                "subset_name": "",
                "subset_name_pinyin": "",
                "used_name": "",
                "birthday": "",
                "birth_place_aboard": 0,
                "birth_place": [],
                "occupation": "",
                "occupation_name": "",
                "operator_ame": "",
                "child_has_hk_id": [],
                "hk_identity_card": "",
                "sex": 0,
                "married": 0,
                "nows": 0,
                "address": [],
                "child_stepchild": [],
                "child_in_hk": [],
                "country_identity": [],
                "hk_identity": "",
                "hk_identity_other": "",
                "live_country": "",
                "is_handled": 0,
                "finish_follow": 0,
                "step": 0,
                "not_fill_num": -1,
                "certificates": [],
                "is_live_oversea_year": 0
            ],
            "spouse": [
                "id": 68416,
                "deceased": 0,
                "custody": 100,
                "nationality": "中国-China",
                "order_id": 133971,
                "relation": "3",
                "has_degree": "1",
                "has_degree_json": [[
                    "degree": "1",
                    "address": "1"
                ]],
                "follow": 1,
                "statement": 0,
                "subset_name": "林源芳",
                "subset_name_pinyin": [
                    "family_name": "LIN",
                    "given_name": "YUANFANG"
                ],
                "used_name": "",
                "birthday": "2012-01-01",
                "birth_place_aboard": 0,
                "birth_place": [
                    "area": ["广东省", "深圳市"],
                    "country": "",
                    "details": "",
                    "foreign": ""
                ],
                "occupation": "在职",
                "occupation_name": "工程师",
                "operator_ame": "",
                "child_has_hk_id": [
                    "has": "N"
                ],
                "hk_identity_card": "",
                "sex": 2,
                "married": "未婚",
                "nows": 0,
                "address": [
                    "area": ["广东省", "深圳市"],
                    "country": "中国-China",
                    "details": "大新地铁站",
                    "foreign": ""
                ],
                "child_stepchild": [
                    "has": "",
                    "info": ""
                ],
                "child_in_hk": [
                    "has": "N"
                ],
                "country_identity": [],
                "hk_identity": "",
                "hk_identity_other": "",
                "live_country": "",
                "is_handled": 0,
                "finish_follow": 0,
                "step": 3,
                "not_fill_num": 0,
                "is_live_oversea_year": 0,
                "surname": "",
                "certificates": [
                    "cnIdentityCard": [
                        "id": 116871,
                        "number": "421087199005010811",
                        "issue_at": "深圳",
                        "issue_date_start_at": "2017-01-01",
                        "issue_date_end_at": "2029-01-01",
                        "img_front": "",
                        "img_back": ""
                    ],
                    "passport": [
                        "id": 116872,
                        "number": "",
                        "issue_at": "",
                        "issue_date_start_at": "",
                        "issue_date_end_at": "",
                        "img_front": "",
                        "img_back": "",
                        "passport_type": 11
                    ]
                ]
            ]
        ] as [String : Any]

        familyMemberInfo = YHFamilyMemberInfo.deserialize(dict: dic)
        loadFamilyMember()
        
    }
    
    func loadFamilyMember() {
        
        items.removeAll()
        
        // 配偶section
        if let sponse = familyMemberInfo?.spouse {
            let sponseArray:[YHFamilyMemberProtocol] = [YHFamilyMemberFormItem(type: .sponse), sponse]
            items.append(sponseArray)
        }
        
        // 父母section
        var parents = [YHFamilyMemberProtocol]()
        if let father = familyMemberInfo?.father {
            parents.append(father)
        }
        if let mother = familyMemberInfo?.mother {
            parents.append(mother)
        }
        if !parents.isEmpty {
            let item0 = YHFamilyMemberFormItem(type: .parent)
            parents.insert(item0, at: 0)
            items.append(parents)
        }

        // 子女section
        var childArr:[YHFamilyMemberProtocol] = [YHFamilyMemberFormItem(type: .child), YHFamilyMemberFormItem(type: .addChild)]
        if let childs = familyMemberInfo?.child, !childs.isEmpty {
            childArr.insert(contentsOf: childs, at:1)
        }
        items.append(childArr)
        
        // 兄妹section
        var brotherArr:[YHFamilyMemberProtocol] = [YHFamilyMemberFormItem(type: .brother), YHFamilyMemberFormItem(type: .addBrother)]
        if let brothers = familyMemberInfo?.brother, !brothers.isEmpty {
            brotherArr.insert(contentsOf: brothers, at:1)
        }
        items.append(brotherArr)
        
        let childCount = familyMemberInfo?.child?.count ?? 0
        if isChildsEditMode == .none && childCount > 0 {
            isChildsEditMode = .canEdit
        }
        
        let brotherCount = familyMemberInfo?.brother?.count ?? 0
        if isBrothersEditMode == .none && brotherCount > 0 {
            isBrothersEditMode = .canEdit
        }
        tableView.reloadData()
    }
}

extension YHFamilyMemberFormVC {
    func addBrother() {
        
        let dict = [
            "id": 68470,
            "deceased": 0,
            "custody": 100,
            "nationality": "",
            "order_id": 133971,
            "relation": "5",
            "has_degree": "100",
            "has_degree_json": [],
            "follow": 0,
            "statement": 0,
            "subset_name": "",
            "subset_name_pinyin": [
                "family_name": "",
                "given_name": ""
            ],
            "used_name": "",
            "birthday": "",
            "birth_place_aboard": 100,
            "birth_place": [
                "area": [],
                "country": "",
                "details": "",
                "foreign": ""
            ],
            "occupation": "",
            "occupation_name": "",
            "operator_ame": "",
            "child_has_hk_id": [
                "has": "",
                "info": ""
            ],
            "hk_identity_card": "",
            "sex": 0,
            "married": "未婚",
            "nows": 100,
            "address": [
                "area": [],
                "country": "",
                "details": "",
                "foreign": ""
            ],
            "child_stepchild": [
                "has": "",
                "info": ""
            ],
            "child_in_hk": [
                "has": "",
                "info": ""
            ],
            "country_identity": [],
            "hk_identity": "",
            "hk_identity_other": "",
            "live_country": "",
            "is_handled": 100,
            "finish_follow": 0,
            "step": 0,
            "not_fill_num": 9,
            "is_live_oversea_year": 0,
            "surname": "",
            "certificates": []
        ] as [String : Any]
        
        let brother:YHBrother = YHBrother.deserialize(dict: dict)!
        
        if var brothers = familyMemberInfo?.brother {
            brothers.append(brother)
            familyMemberInfo!.brother = brothers
        } else {
            familyMemberInfo?.brother = [brother]
        }
        loadFamilyMember()
    }
    
    func addChild() {
        let dict = [
            "id": 68493,
            "deceased": 0,
            "custody": 100,
            "nationality": "",
            "order_id": 133971,
            "relation": "4",
            "has_degree": "100",
            "has_degree_json": [],
            "follow": 0,
            "statement": 0,
            "subset_name": "",
            "subset_name_pinyin": [
                "family_name": "",
                "given_name": ""
            ],
            "used_name": "",
            "birthday": "",
            "birth_place_aboard": 100,
            "birth_place": [
                "area": [],
                "country": "",
                "details": "",
                "foreign": ""
            ],
            "occupation": "",
            "occupation_name": "",
            "operator_ame": "",
            "child_has_hk_id": [
                "has": "",
                "info": ""
            ],
            "hk_identity_card": "",
            "sex": 0,
            "married": "未婚",
            "nows": 100,
            "address": [
                "area": [],
                "country": "",
                "details": "",
                "foreign": ""
            ],
            "child_stepchild": [
                "has": "",
                "info": ""
            ],
            "child_in_hk": [
                "has": "",
                "info": ""
            ],
            "country_identity": [],
            "hk_identity": "",
            "hk_identity_other": "",
            "live_country": "",
            "is_handled": 100,
            "finish_follow": 0,
            "step": 0,
            "not_fill_num": 10,
            "is_live_oversea_year": 0,
            "surname": "",
            "certificates":[]
           ] as [String : Any]
        
        let child = YHChild.deserialize(dict: dict)!
        if var childs = familyMemberInfo?.child {
            childs.append(child)
            familyMemberInfo!.child = childs
        } else {
            familyMemberInfo?.child = [child]
        }
        loadFamilyMember()
    }
}

extension YHFamilyMemberFormVC : UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return items.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section >= items.count { return 0 }
        let arr = items[section]
        return arr.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let arr = items[indexPath.section]
        let item:YHFamilyMemberProtocol = arr[indexPath.row]
        
        if item is YHFamilyMemberFormItem { // 是标题或添加项目
            
            let formItem = item as! YHFamilyMemberFormItem
            if formItem.type == .addChild || formItem.type == .addBrother { // 新增子女/兄弟姐妹
                
                let cell = tableView.dequeueReusableCell(withIdentifier: YHFormAddInfoCell.cellReuseIdentifier, for: indexPath) as! YHFormAddInfoCell
                createCorner(cell: cell, arr: arr, indexPath: indexPath)
                cell.clickBlock = { [weak self] in
                    if formItem.type == .addBrother {
                        self?.addBrother()
                    } else if formItem.type == .addChild {
                        self?.addChild()
                    }
                 }
                return cell
            }
            // 配偶父母子女兄妹title
            let cell = tableView.dequeueReusableCell(withIdentifier: YHFormTitleCell.cellReuseIdentifier, for: indexPath) as! YHFormTitleCell
            createCorner(cell: cell, arr: arr, indexPath: indexPath)
            cell.titleLabel.text = formItem.getTitle()
            cell.subTitleLabel.text = formItem.getSubTitle()
            
            // 决定右边按钮显示样式
            cell.showEditType(type: .none)
            if formItem.type == .child {
                cell.showEditType(type: isChildsEditMode)
                
            } else if formItem.type == .brother {
                cell.showEditType(type: isBrothersEditMode)
            }
            
            // 右边按钮点击事件处理
            cell.rightClickBlock = { [weak self] type in
                if formItem.type == .child {
                    if self?.isChildsEditMode == .canCancel {
                        self?.isChildsEditMode = .canEdit
                    } else if self?.isChildsEditMode == .canEdit {
                        self?.isChildsEditMode = .canCancel
                    }
                }
                if formItem.type == .brother {
                    if self?.isBrothersEditMode == .canCancel {
                        self?.isBrothersEditMode = .canEdit
                    } else if self?.isBrothersEditMode == .canEdit {
                        self?.isBrothersEditMode = .canCancel
                    }
                }
                self?.loadFamilyMember()
            }
            return cell
        }
    
        // 配偶父母子女兄妹具体条目
        if item is YHParent || item is YHChild || item is YHSpouse || item is YHBrother {
            
            let cell = tableView.dequeueReusableCell(withIdentifier: YHSingleLineSelectCell.cellReuseIdentifier, for: indexPath) as! YHSingleLineSelectCell
            createCorner(cell: cell, arr: arr, indexPath: indexPath)
            cell.isShowDeleteBtn = false
            cell.deleteBlock = nil
            if item is YHChild || item is YHBrother {
                
                if item is YHChild {
                    cell.isShowDeleteBtn = (isChildsEditMode == .canCancel)
                } else if item is YHBrother {
                    cell.isShowDeleteBtn = (isBrothersEditMode == .canCancel)
                }

                // 删除子女兄妹处理
                cell.deleteBlock = { [weak self] in
                    if item is YHChild {
                        let childCount = self?.familyMemberInfo?.child?.count ?? 0
                        if (indexPath.row < childCount) {
                            // 因为子女兄妹section第一行都是标题所以row-1
                            self?.familyMemberInfo?.child?.remove(at: indexPath.row-1)
                            self?.isChildsEditMode = .none
                            self?.isBrothersEditMode = .none
                        }
                    } else if item is YHBrother {
                        
                            let brotherCount = self?.familyMemberInfo?.brother?.count ?? 0
                            if (indexPath.row < brotherCount) {
                                // 因为子女兄妹section第一行都是标题所以row-1
                                self?.familyMemberInfo?.brother?.remove(at: indexPath.row-1)
                                self?.isChildsEditMode = .none
                                self?.isBrothersEditMode = .none
                            }
                    }
                    self?.loadFamilyMember()
                }
            }
            
            return cell
        }

        let cell = tableView.dequeueReusableCell(withIdentifier: "UITableViewCell", for: indexPath)
        createCorner(cell: cell, arr: arr, indexPath: indexPath)
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        
        let arr = items[indexPath.section]
        let item:YHFamilyMemberProtocol = arr[indexPath.row]
        
        if item is YHFamilyMemberFormItem { // 是标题或添加项目
            let titleItem = item as! YHFamilyMemberFormItem
            if titleItem.type == .addChild || titleItem.type == .addBrother {
                return 77.0
            }
            return 52.0
        }
        return 64.0
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        
        return UIView()
    }
    
    private func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> CGFloat {
        return 14.0
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
         let vc = YHSpouseFormVC()
         self.navigationController?.pushViewController(vc)
    }
        
    func createCorner(cell:UITableViewCell, arr:Array<Any>, indexPath:IndexPath) {
        // 设置每块section圆角
        if (indexPath.row == 0) {
            let corner = UIRectCorner(rawValue: UIRectCorner.topLeft.rawValue | UIRectCorner.topRight.rawValue)
            cell.createCorner(CGSizeMake(12.0, 12.0), corner)
            
        } else if (indexPath.row == arr.count-1) {
            let corner = UIRectCorner(rawValue: UIRectCorner.bottomLeft.rawValue | UIRectCorner.bottomRight.rawValue)
            cell.createCorner(CGSizeMake(12.0, 12.0), corner)
            
        } else {
            cell.layer.mask = nil
        }
    }
}

